/* eslint-disable no-console */
import type { HighlighterCore, LanguageRegistration } from '@shikijs/types'
import { createHighlighterCore, normalizeTheme } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import githubDark from 'shiki/dist/themes/github-dark.mjs'

// Import languages
import css from 'shiki/dist/langs/css.mjs'
import html from 'shiki/dist/langs/html.mjs'
import javascript from 'shiki/dist/langs/javascript.mjs'
import json from 'shiki/dist/langs/json.mjs'
import markdown from 'shiki/dist/langs/markdown.mjs'
import php from 'shiki/dist/langs/php.mjs'
import tsx from 'shiki/dist/langs/tsx.mjs'
import typescript from 'shiki/dist/langs/typescript.mjs'

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args)
  }
}

export type AvailableLanguage = 'php' | 'tsx' | 'typescript' | 'javascript' | 'json' | 'html' | 'css' | 'markdown' | 'plaintext'

// Define raw languages with proper typing
const rawLanguages: Record<string, any> = {
  typescript,
  javascript,
  json,
  html,
  css,
  markdown,
  tsx,
  php,
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{ match: '.*', name: 'text.plain' }],
    },
  },
}

let highlighterInstance: HighlighterCore | null = null

// Process raw languages into LanguageRegistrations
function processLanguages(): LanguageRegistration[] {
  return Object.entries(rawLanguages).map(([key, lang]) => ({
    name: lang.id || key,
    scopeName: lang.scopeName,
    patterns: lang.patterns || [],
    repository: lang.repository || {},
    aliases: lang.aliases || [],
    embeddedLangs: lang.embeddedLanguages || [],
    grammar: lang.grammar || {},
  }))
}

export async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance')
    return highlighterInstance
  }

  try {
    log('Creating new highlighter instance...')

    // Create engine
    const engine = createJavaScriptRegexEngine()

    // Process languages
    const languages = processLanguages()

    // Create highlighter
    highlighterInstance = await createHighlighterCore({
      themes: [normalizeTheme(githubDark)],
      langs: languages,
      engine,
    })

    if (!highlighterInstance) {
      throw new Error('Failed to create highlighter instance')
    }

    const loadedLangs = highlighterInstance.getLoadedLanguages()
    log('Successfully loaded languages:', loadedLangs)

    return highlighterInstance
  }
  catch (error) {
    log('Error creating highlighter:', error)
    throw error
  }
}

export function isValidLanguage(lang: string): lang is AvailableLanguage {
  const normalized = lang.toLowerCase()
  return Object.keys(rawLanguages).includes(normalized)
}

export function getSafeLanguage(lang: string): AvailableLanguage {
  const normalized = lang.toLowerCase()
  if (isValidLanguage(normalized)) {
    return normalized as AvailableLanguage
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`)
  return 'plaintext'
}
